/* (c) Copyright Daniel D. Lanciani 1998.  All rights reserved. */

#include <windows.h>

static HANDLE _ndis3vh = (HANDLE)-1;
static char copyright[] = "Copyright Daniel Lanciani 1998";

#define BAD_HANDLE 1
#define NO_CLASS 2
#define NO_TYPE 3
#define NO_NUMBER 4
#define BAD_TYPE 5
#define NO_MULTICAST 6
#define CANT_TERMINATE 7
#define BAD_MODE 8
#define NO_SPACE 9
#define TYPE_INUSE 10
#define BAD_COMMAND 11
#define CANT_SEND 12
#define CANT_SET 13
#define BAD_ADDRESS 14
#define CANT_RESET 15
#define WOULD_BLOCK 200
#define CANCELED 201

#define RECV_FLAG_NOBLOCK 1
#define RECV_FLAG_PEEK 2

/*
Generic function to call ndis3pkt's DeviceIoControl entry.
*/
static long
nd_call(arg0)
{
	long res[1];
	int os;

	*res = -1;
	DeviceIoControl(_ndis3vh, 200, &arg0, 7 * sizeof(long),
		res, sizeof(res), &os, 0);
	return(*res);
}

/*
Open device and return non-zero if ndis3pkt is available.
*/
nd_loaded()
{
	if(_ndis3vh == (HANDLE)-1)
		_ndis3vh = CreateFile("\\\\.\\NDIS3PKT", 0, 0,
			NULL, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);
	if(_ndis3vh == (HANDLE)-1)
		return(0);
	return(1);
}

/*
Return class and Windows name of a given unit.
*/
nd_driver_info(unit, class, name)
int *class;
char *name;
{
	return(nd_call(1, unit, class, name, (int *)0));
}

/*
Allocate handle for type.  At most qlen packets will be
queued on this handle for later nd_recv().
*/
nd_access_type(unit, handle, type, typelen, qlen)
int *handle;
char *type;
{
	return(nd_call(2, unit, handle, type, typelen, qlen));
}

/*
Free handle.
*/
nd_release_type(unit, handle)
{
	return(nd_call(3, unit, handle));
}

/*
Send complete frame of len bytes at data.
*/
nd_send_pkt(unit, data, len)
char *data;
{
	return(nd_call(4, unit, data, len));
}

/*
Shut down driver.
*/
nd_terminate(unit)
{
	return(nd_call(5, unit));
}

/*
Copy MAC address of unit to buf.  Size should be at least 6.
*/
nd_get_address(unit, size, buf)
char *buf;
{
	return(nd_call(6, unit, size, buf));
}

/*
Set receiver mode.
*/
nd_set_rcv_mode(unit, handle, mode)
{
	return(nd_call(20, unit, handle, mode));
}

/*
Get receiver mode.
*/
nd_get_rcv_mode(unit, handle, mode)
int *mode;
{
	return(nd_call(21, unit, handle, mode));
}

/*
Read up to *cnt bytes of packet from handle into buf.  If flags
includes RECV_FLAG_NOBLOCK then nd_recv will not sleep and the
return code will be WOULD_BLOCK if no packets are immediately
available.  If flags includes RECV_FLAG_PEEK then the returned
packet will not be removed from the queue.  The number of bytes
copied is returned in *cnt.
*/
nd_recv(unit, handle, buf, cnt, flags)
char *buf;
int *cnt;
{
	return(nd_call(200, unit, handle, buf, cnt, flags));
}

/*
Cancel all blocking nd_recv()s associated with handle.  They
will return the CANCELED error code.
*/
nd_cancel_recv(unit, handle)
{
	return(nd_call(201, unit, handle));
}

#include <stdio.h>

main(argc, argv)
char **argv;
{
	register int i;
	int m, h;
	unsigned char buf[1514];

	if(!nd_loaded()) {
		fprintf(stderr, "Ndis3pkt not loaded\n");
		exit(1);
	}
	if(argc > 1) {
		printf("cancel_recv = %d\n", nd_cancel_recv(0, atoi(argv[1])));
		exit(0);
	}
	printf("driver_info = %d\n", nd_driver_info(0, &m, buf));
	printf("class = %d, Windowsname = %s\n", m, buf);
	printf("get_address = %d\n", nd_get_address(0, 6, buf));
	printf("address = ");
	for(i = 0; i < 6; i++) {
		printf("%02x", buf[i] & 0xff);
		if(i == 5)
			printf("\n");
		else
			printf(":");
	}
	printf("get_rcv_mode = %d\n", nd_get_rcv_mode(0, -1, &m));
	printf("mode = %d\n", m);
	printf("access_type = %d\n", nd_access_type(0, &h, "", 0, 1));
	printf("handle = %d\n", h);
	printf("set_rcv_mode = %d\n", nd_set_rcv_mode(0, h, 6));
	for(i = 0; i < 100; i++)
		buf[i] = i;
	printf("send = %d\n", nd_send_pkt(0, buf, sizeof(buf)));
	while(!kbhit()) {
		m = sizeof(buf);
		printf("recv = %d\n", nd_recv(0, h, buf, &m, 0));
		printf("size recv = %d\n", m);
		printf("data: ");
		for(i = 0; i < 20; i++)
			printf("%02x ", buf[i]);
		printf("\n");
	}
	printf("set_rcv_mode = %d\n", nd_set_rcv_mode(0, h, 3));
	printf("release_type = %d\n", nd_release_type(0, h));
}
